# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/load_balancing_policies/subset/v3/subset.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/load_balancing_policies/subset/v3/subset.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.cluster.v3 import cluster_pb2 as envoy_dot_config_dot_cluster_dot_v3_dot_cluster__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n?envoy/extensions/load_balancing_policies/subset/v3/subset.proto\x12\x32\x65nvoy.extensions.load_balancing_policies.subset.v3\x1a%envoy/config/cluster/v3/cluster.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xb4\t\n\x06Subset\x12t\n\x0f\x66\x61llback_policy\x18\x01 \x01(\x0e\x32Q.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetFallbackPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12/\n\x0e\x64\x65\x66\x61ult_subset\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x65\n\x10subset_selectors\x18\x03 \x03(\x0b\x32K.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetSelector\x12\x1c\n\x14\x61llow_redundant_keys\x18\n \x01(\x08\x12\x1d\n\x15locality_weight_aware\x18\x04 \x01(\x08\x12\x1d\n\x15scale_locality_weight\x18\x05 \x01(\x08\x12\x16\n\x0epanic_mode_any\x18\x06 \x01(\x08\x12\x13\n\x0blist_as_any\x18\x07 \x01(\x08\x12\x85\x01\n\x18metadata_fallback_policy\x18\x08 \x01(\x0e\x32Y.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetMetadataFallbackPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12P\n\x10subset_lb_policy\x18\t \x01(\x0b\x32,.envoy.config.cluster.v3.LoadBalancingPolicyB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x1a\xe9\x02\n\x10LbSubsetSelector\x12\x0c\n\x04keys\x18\x01 \x03(\t\x12\x1e\n\x16single_host_per_subset\x18\x04 \x01(\x08\x12\x8d\x01\n\x0f\x66\x61llback_policy\x18\x02 \x01(\x0e\x32j.envoy.extensions.load_balancing_policies.subset.v3.Subset.LbSubsetSelector.LbSubsetSelectorFallbackPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x1c\n\x14\x66\x61llback_keys_subset\x18\x03 \x03(\t\"y\n\x1eLbSubsetSelectorFallbackPolicy\x12\x0f\n\x0bNOT_DEFINED\x10\x00\x12\x0f\n\x0bNO_FALLBACK\x10\x01\x12\x10\n\x0c\x41NY_ENDPOINT\x10\x02\x12\x12\n\x0e\x44\x45\x46\x41ULT_SUBSET\x10\x03\x12\x0f\n\x0bKEYS_SUBSET\x10\x04\"O\n\x16LbSubsetFallbackPolicy\x12\x0f\n\x0bNO_FALLBACK\x10\x00\x12\x10\n\x0c\x41NY_ENDPOINT\x10\x01\x12\x12\n\x0e\x44\x45\x46\x41ULT_SUBSET\x10\x02\"M\n\x1eLbSubsetMetadataFallbackPolicy\x12\x18\n\x14METADATA_NO_FALLBACK\x10\x00\x12\x11\n\rFALLBACK_LIST\x10\x01:-\x9a\xc5\x88\x1e(\n&envoy.config.cluster.v3.LbSubsetConfigB\xbd\x01\n@io.envoyproxy.envoy.extensions.load_balancing_policies.subset.v3B\x0bSubsetProtoP\x01Zbgithub.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/subset/v3;subsetv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.load_balancing_policies.subset.v3.subset_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.load_balancing_policies.subset.v3B\013SubsetProtoP\001Zbgithub.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/subset/v3;subsetv3\272\200\310\321\006\002\020\002'
  _globals['_SUBSET_LBSUBSETSELECTOR'].fields_by_name['fallback_policy']._loaded_options = None
  _globals['_SUBSET_LBSUBSETSELECTOR'].fields_by_name['fallback_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_SUBSET'].fields_by_name['fallback_policy']._loaded_options = None
  _globals['_SUBSET'].fields_by_name['fallback_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_SUBSET'].fields_by_name['metadata_fallback_policy']._loaded_options = None
  _globals['_SUBSET'].fields_by_name['metadata_fallback_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_SUBSET'].fields_by_name['subset_lb_policy']._loaded_options = None
  _globals['_SUBSET'].fields_by_name['subset_lb_policy']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_SUBSET']._loaded_options = None
  _globals['_SUBSET']._serialized_options = b'\232\305\210\036(\n&envoy.config.cluster.v3.LbSubsetConfig'
  _globals['_SUBSET']._serialized_start=280
  _globals['_SUBSET']._serialized_end=1484
  _globals['_SUBSET_LBSUBSETSELECTOR']._serialized_start=916
  _globals['_SUBSET_LBSUBSETSELECTOR']._serialized_end=1277
  _globals['_SUBSET_LBSUBSETSELECTOR_LBSUBSETSELECTORFALLBACKPOLICY']._serialized_start=1156
  _globals['_SUBSET_LBSUBSETSELECTOR_LBSUBSETSELECTORFALLBACKPOLICY']._serialized_end=1277
  _globals['_SUBSET_LBSUBSETFALLBACKPOLICY']._serialized_start=1279
  _globals['_SUBSET_LBSUBSETFALLBACKPOLICY']._serialized_end=1358
  _globals['_SUBSET_LBSUBSETMETADATAFALLBACKPOLICY']._serialized_start=1360
  _globals['_SUBSET_LBSUBSETMETADATAFALLBACKPOLICY']._serialized_end=1437
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha/tra.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha/tra.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.config.core.v3 import grpc_service_pb2 as envoy_dot_config_dot_core_dot_v3_dot_grpc__service__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nHcontrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha/tra.proto\x12\x36\x65nvoy.extensions.filters.network.sip_proxy.tra.v3alpha\x1a(envoy/config/core/v3/config_source.proto\x1a\'envoy/config/core/v3/grpc_service.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xcc\x01\n\x10TraServiceConfig\x12\x41\n\x0cgrpc_service\x18\x01 \x01(\x0b\x32!.envoy.config.core.v3.GrpcServiceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12I\n\x15transport_api_version\x18\x02 \x01(\x0e\x32 .envoy.config.core.v3.ApiVersionB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12*\n\x07timeout\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\"\x9b\x04\n\x11TraServiceRequest\x12\x0c\n\x04type\x18\x01 \x01(\t\x12_\n\x0e\x63reate_request\x18\x02 \x01(\x0b\x32\x45.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateRequestH\x00\x12_\n\x0eupdate_request\x18\x03 \x01(\x0b\x32\x45.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateRequestH\x00\x12\x63\n\x10retrieve_request\x18\x04 \x01(\x0b\x32G.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveRequestH\x00\x12_\n\x0e\x64\x65lete_request\x18\x05 \x01(\x0b\x32\x45.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.DeleteRequestH\x00\x12\x65\n\x11subscribe_request\x18\x06 \x01(\x0b\x32H.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.SubscribeRequestH\x00\x42\t\n\x07request\"\xc4\x04\n\x12TraServiceResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\x12\x0b\n\x03ret\x18\x02 \x01(\x05\x12\x0e\n\x06reason\x18\x03 \x01(\t\x12\x61\n\x0f\x63reate_response\x18\x04 \x01(\x0b\x32\x46.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateResponseH\x00\x12\x61\n\x0fupdate_response\x18\x05 \x01(\x0b\x32\x46.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateResponseH\x00\x12\x65\n\x11retrieve_response\x18\x06 \x01(\x0b\x32H.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveResponseH\x00\x12\x61\n\x0f\x64\x65lete_response\x18\x07 \x01(\x0b\x32\x46.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.DeleteResponseH\x00\x12g\n\x12subscribe_response\x18\x08 \x01(\x0b\x32I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.SubscribeResponseH\x00\x42\n\n\x08response\"\xb0\x02\n\rCreateRequest\x12]\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32O.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateRequest.DataEntry\x12\x63\n\x07\x63ontext\x18\x02 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.CreateRequest.ContextEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x10\n\x0e\x43reateResponse\"\xb0\x02\n\rUpdateRequest\x12]\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32O.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateRequest.DataEntry\x12\x63\n\x07\x63ontext\x18\x02 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.UpdateRequest.ContextEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x10\n\x0eUpdateResponse\"\xb5\x01\n\x0fRetrieveRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x65\n\x07\x63ontext\x18\x02 \x03(\x0b\x32T.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveRequest.ContextEntry\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xa1\x01\n\x10RetrieveResponse\x12`\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.RetrieveResponse.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\xb1\x01\n\rDeleteRequest\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x63\n\x07\x63ontext\x18\x02 \x03(\x0b\x32R.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.DeleteRequest.ContextEntry\x1a.\n\x0c\x43ontextEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"\x10\n\x0e\x44\x65leteResponse\"\x12\n\x10SubscribeRequest\"\xa3\x01\n\x11SubscribeResponse\x12\x61\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32S.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.SubscribeResponse.DataEntry\x1a+\n\tDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x32\xc7\x06\n\nTraService\x12\xa1\x01\n\x06\x43reate\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa1\x01\n\x06Update\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa3\x01\n\x08Retrieve\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa1\x01\n\x06\x44\x65lete\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x12\xa6\x01\n\tSubscribe\x12I.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceRequest\x1aJ.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.TraServiceResponse\"\x00\x30\x01\x42\xc1\x01\nDio.envoyproxy.envoy.extensions.filters.network.sip_proxy.tra.v3alphaB\x08TraProtoP\x01Zegithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.network.sip_proxy.tra.v3alpha.tra_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nDio.envoyproxy.envoy.extensions.filters.network.sip_proxy.tra.v3alphaB\010TraProtoP\001Zegithub.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/network/sip_proxy/tra/v3alpha\272\200\310\321\006\002\020\002'
  _globals['_TRASERVICECONFIG'].fields_by_name['grpc_service']._loaded_options = None
  _globals['_TRASERVICECONFIG'].fields_by_name['grpc_service']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_TRASERVICECONFIG'].fields_by_name['transport_api_version']._loaded_options = None
  _globals['_TRASERVICECONFIG'].fields_by_name['transport_api_version']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_CREATEREQUEST_DATAENTRY']._loaded_options = None
  _globals['_CREATEREQUEST_DATAENTRY']._serialized_options = b'8\001'
  _globals['_CREATEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_CREATEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_UPDATEREQUEST_DATAENTRY']._loaded_options = None
  _globals['_UPDATEREQUEST_DATAENTRY']._serialized_options = b'8\001'
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_RETRIEVERESPONSE_DATAENTRY']._loaded_options = None
  _globals['_RETRIEVERESPONSE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_DELETEREQUEST_CONTEXTENTRY']._loaded_options = None
  _globals['_DELETEREQUEST_CONTEXTENTRY']._serialized_options = b'8\001'
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._loaded_options = None
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._serialized_options = b'8\001'
  _globals['_TRASERVICECONFIG']._serialized_start=304
  _globals['_TRASERVICECONFIG']._serialized_end=508
  _globals['_TRASERVICEREQUEST']._serialized_start=511
  _globals['_TRASERVICEREQUEST']._serialized_end=1050
  _globals['_TRASERVICERESPONSE']._serialized_start=1053
  _globals['_TRASERVICERESPONSE']._serialized_end=1633
  _globals['_CREATEREQUEST']._serialized_start=1636
  _globals['_CREATEREQUEST']._serialized_end=1940
  _globals['_CREATEREQUEST_DATAENTRY']._serialized_start=1849
  _globals['_CREATEREQUEST_DATAENTRY']._serialized_end=1892
  _globals['_CREATEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_CREATEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_CREATERESPONSE']._serialized_start=1942
  _globals['_CREATERESPONSE']._serialized_end=1958
  _globals['_UPDATEREQUEST']._serialized_start=1961
  _globals['_UPDATEREQUEST']._serialized_end=2265
  _globals['_UPDATEREQUEST_DATAENTRY']._serialized_start=1849
  _globals['_UPDATEREQUEST_DATAENTRY']._serialized_end=1892
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_UPDATEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_UPDATERESPONSE']._serialized_start=2267
  _globals['_UPDATERESPONSE']._serialized_end=2283
  _globals['_RETRIEVEREQUEST']._serialized_start=2286
  _globals['_RETRIEVEREQUEST']._serialized_end=2467
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_RETRIEVEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_RETRIEVERESPONSE']._serialized_start=2470
  _globals['_RETRIEVERESPONSE']._serialized_end=2631
  _globals['_RETRIEVERESPONSE_DATAENTRY']._serialized_start=1849
  _globals['_RETRIEVERESPONSE_DATAENTRY']._serialized_end=1892
  _globals['_DELETEREQUEST']._serialized_start=2634
  _globals['_DELETEREQUEST']._serialized_end=2811
  _globals['_DELETEREQUEST_CONTEXTENTRY']._serialized_start=1894
  _globals['_DELETEREQUEST_CONTEXTENTRY']._serialized_end=1940
  _globals['_DELETERESPONSE']._serialized_start=2813
  _globals['_DELETERESPONSE']._serialized_end=2829
  _globals['_SUBSCRIBEREQUEST']._serialized_start=2831
  _globals['_SUBSCRIBEREQUEST']._serialized_end=2849
  _globals['_SUBSCRIBERESPONSE']._serialized_start=2852
  _globals['_SUBSCRIBERESPONSE']._serialized_end=3015
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._serialized_start=1849
  _globals['_SUBSCRIBERESPONSE_DATAENTRY']._serialized_end=1892
  _globals['_TRASERVICE']._serialized_start=3018
  _globals['_TRASERVICE']._serialized_end=3857
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/tracing/v3/custom_tag.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/tracing/v3/custom_tag.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.type.metadata.v3 import metadata_pb2 as envoy_dot_type_dot_metadata_dot_v3_dot_metadata__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&envoy/type/tracing/v3/custom_tag.proto\x12\x15\x65nvoy.type.tracing.v3\x1a%envoy/type/metadata/v3/metadata.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xe0\x06\n\tCustomTag\x12\x14\n\x03tag\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12;\n\x07literal\x18\x02 \x01(\x0b\x32(.envoy.type.tracing.v3.CustomTag.LiteralH\x00\x12\x43\n\x0b\x65nvironment\x18\x03 \x01(\x0b\x32,.envoy.type.tracing.v3.CustomTag.EnvironmentH\x00\x12\x41\n\x0erequest_header\x18\x04 \x01(\x0b\x32\'.envoy.type.tracing.v3.CustomTag.HeaderH\x00\x12=\n\x08metadata\x18\x05 \x01(\x0b\x32).envoy.type.tracing.v3.CustomTag.MetadataH\x00\x12\x0f\n\x05value\x18\x06 \x01(\tH\x00\x1aQ\n\x07Literal\x12\x16\n\x05value\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:.\x9a\xc5\x88\x1e)\n\'envoy.type.tracing.v2.CustomTag.Literal\x1ao\n\x0b\x45nvironment\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x15\n\rdefault_value\x18\x02 \x01(\t:2\x9a\xc5\x88\x1e-\n+envoy.type.tracing.v2.CustomTag.Environment\x1ak\n\x06Header\x12\x1b\n\x04name\x18\x01 \x01(\tB\r\xfa\x42\nr\x08\x10\x01\xc0\x01\x01\xc8\x01\x00\x12\x15\n\rdefault_value\x18\x02 \x01(\t:-\x9a\xc5\x88\x1e(\n&envoy.type.tracing.v2.CustomTag.Header\x1a\xc1\x01\n\x08Metadata\x12\x32\n\x04kind\x18\x01 \x01(\x0b\x32$.envoy.type.metadata.v3.MetadataKind\x12\x39\n\x0cmetadata_key\x18\x02 \x01(\x0b\x32#.envoy.type.metadata.v3.MetadataKey\x12\x15\n\rdefault_value\x18\x03 \x01(\t:/\x9a\xc5\x88\x1e*\n(envoy.type.tracing.v2.CustomTag.Metadata:&\x9a\xc5\x88\x1e!\n\x1f\x65nvoy.type.tracing.v2.CustomTagB\x0b\n\x04type\x12\x03\xf8\x42\x01\x42\x87\x01\n#io.envoyproxy.envoy.type.tracing.v3B\x0e\x43ustomTagProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/tracing/v3;tracingv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.tracing.v3.custom_tag_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#io.envoyproxy.envoy.type.tracing.v3B\016CustomTagProtoP\001ZFgithub.com/envoyproxy/go-control-plane/envoy/type/tracing/v3;tracingv3\272\200\310\321\006\002\020\002'
  _globals['_CUSTOMTAG_LITERAL'].fields_by_name['value']._loaded_options = None
  _globals['_CUSTOMTAG_LITERAL'].fields_by_name['value']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CUSTOMTAG_LITERAL']._loaded_options = None
  _globals['_CUSTOMTAG_LITERAL']._serialized_options = b'\232\305\210\036)\n\'envoy.type.tracing.v2.CustomTag.Literal'
  _globals['_CUSTOMTAG_ENVIRONMENT'].fields_by_name['name']._loaded_options = None
  _globals['_CUSTOMTAG_ENVIRONMENT'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CUSTOMTAG_ENVIRONMENT']._loaded_options = None
  _globals['_CUSTOMTAG_ENVIRONMENT']._serialized_options = b'\232\305\210\036-\n+envoy.type.tracing.v2.CustomTag.Environment'
  _globals['_CUSTOMTAG_HEADER'].fields_by_name['name']._loaded_options = None
  _globals['_CUSTOMTAG_HEADER'].fields_by_name['name']._serialized_options = b'\372B\nr\010\020\001\300\001\001\310\001\000'
  _globals['_CUSTOMTAG_HEADER']._loaded_options = None
  _globals['_CUSTOMTAG_HEADER']._serialized_options = b'\232\305\210\036(\n&envoy.type.tracing.v2.CustomTag.Header'
  _globals['_CUSTOMTAG_METADATA']._loaded_options = None
  _globals['_CUSTOMTAG_METADATA']._serialized_options = b'\232\305\210\036*\n(envoy.type.tracing.v2.CustomTag.Metadata'
  _globals['_CUSTOMTAG'].oneofs_by_name['type']._loaded_options = None
  _globals['_CUSTOMTAG'].oneofs_by_name['type']._serialized_options = b'\370B\001'
  _globals['_CUSTOMTAG'].fields_by_name['tag']._loaded_options = None
  _globals['_CUSTOMTAG'].fields_by_name['tag']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CUSTOMTAG']._loaded_options = None
  _globals['_CUSTOMTAG']._serialized_options = b'\232\305\210\036!\n\037envoy.type.tracing.v2.CustomTag'
  _globals['_CUSTOMTAG']._serialized_start=196
  _globals['_CUSTOMTAG']._serialized_end=1060
  _globals['_CUSTOMTAG_LITERAL']._serialized_start=508
  _globals['_CUSTOMTAG_LITERAL']._serialized_end=589
  _globals['_CUSTOMTAG_ENVIRONMENT']._serialized_start=591
  _globals['_CUSTOMTAG_ENVIRONMENT']._serialized_end=702
  _globals['_CUSTOMTAG_HEADER']._serialized_start=704
  _globals['_CUSTOMTAG_HEADER']._serialized_end=811
  _globals['_CUSTOMTAG_METADATA']._serialized_start=814
  _globals['_CUSTOMTAG_METADATA']._serialized_end=1007
# @@protoc_insertion_point(module_scope)



# Copyright 2022 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Retrieves supported bazel versions from plaintext file."""

_SUPPORTED_VERSIONS_FILE = "bazel/supported_versions.txt"


def _get_supported_bazel_versions():
    versions = []
    with open(_SUPPORTED_VERSIONS_FILE, "r") as f:
        for line in f:
            versions.append(line.strip())
    return versions


def mako_plugin(dictionary):
    versions = _get_supported_bazel_versions()
    dictionary["supported_bazel_versions"] = versions
    # The entry listed in the "supported_version.txt" file is considered
    # to be our repo-wide bazel version.
    dictionary["primary_bazel_version"] = versions[0]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/api/cloudquotas/v1beta/quota_adjuster_settings.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/api/cloudquotas/v1beta/quota_adjuster_settings.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.api import client_pb2 as google_dot_api_dot_client__pb2
from google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n;google/api/cloudquotas/v1beta/quota_adjuster_settings.proto\x12\x1dgoogle.api.cloudquotas.v1beta\x1a\x1cgoogle/api/annotations.proto\x1a\x17google/api/client.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"i\n\x1fGetQuotaAdjusterSettingsRequest\x12\x46\n\x04name\x18\x01 \x01(\tB8\xe0\x41\x02\xfa\x41\x32\n0cloudquotas.googleapis.com/QuotaAdjusterSettings\"\xd2\x01\n\"UpdateQuotaAdjusterSettingsRequest\x12Z\n\x17quota_adjuster_settings\x18\x01 \x01(\x0b\x32\x34.google.api.cloudquotas.v1beta.QuotaAdjusterSettingsB\x03\xe0\x41\x02\x12\x34\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x03\xe0\x41\x01\x12\x1a\n\rvalidate_only\x18\x03 \x01(\x08\x42\x03\xe0\x41\x01\"\xf6\x04\n\x15QuotaAdjusterSettings\x12\x11\n\x04name\x18\x01 \x01(\tB\x03\xe0\x41\x08\x12X\n\nenablement\x18\x02 \x01(\x0e\x32?.google.api.cloudquotas.v1beta.QuotaAdjusterSettings.EnablementB\x03\xe0\x41\x01\x12\x34\n\x0bupdate_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x03\xe0\x41\x03\x12\x11\n\x04\x65tag\x18\x06 \x01(\tB\x03\xe0\x41\x01\x12\x16\n\tinherited\x18\x07 \x01(\x08\x42\x03\xe0\x41\x01\x12\x1b\n\x0einherited_from\x18\x08 \x01(\tB\x03\xe0\x41\x03\"C\n\nEnablement\x12\x1a\n\x16\x45NABLEMENT_UNSPECIFIED\x10\x00\x12\x0b\n\x07\x45NABLED\x10\x02\x12\x0c\n\x08\x44ISABLED\x10\x03:\xac\x02\xea\x41\xa8\x02\n0cloudquotas.googleapis.com/QuotaAdjusterSettings\x12=projects/{project}/locations/{location}/quotaAdjusterSettings\x12Gorganizations/{organization}/locations/{location}/quotaAdjusterSettings\x12;folders/{folder}/locations/{location}/quotaAdjusterSettings*\x15quotaAdjusterSettings2\x15quotaAdjusterSettingsR\x01\x01\x32\xec\x07\n\x1cQuotaAdjusterSettingsManager\x12\x98\x04\n\x1bUpdateQuotaAdjusterSettings\x12\x41.google.api.cloudquotas.v1beta.UpdateQuotaAdjusterSettingsRequest\x1a\x34.google.api.cloudquotas.v1beta.QuotaAdjusterSettings\"\xff\x02\xda\x41#quota_adjuster_settings,update_mask\x82\xd3\xe4\x93\x02\xd2\x02\x32S/v1beta/{quota_adjuster_settings.name=projects/*/locations/*/quotaAdjusterSettings}:\x17quota_adjuster_settingsZm2R/v1beta/{quota_adjuster_settings.name=folders/*/locations/*/quotaAdjusterSettings}:\x17quota_adjuster_settingsZs2X/v1beta/{quota_adjuster_settings.name=organizations/*/locations/*/quotaAdjusterSettings}:\x17quota_adjuster_settings\x12\xe0\x02\n\x18GetQuotaAdjusterSettings\x12>.google.api.cloudquotas.v1beta.GetQuotaAdjusterSettingsRequest\x1a\x34.google.api.cloudquotas.v1beta.QuotaAdjusterSettings\"\xcd\x01\xda\x41\x04name\x82\xd3\xe4\x93\x02\xbf\x01\x12;/v1beta/{name=projects/*/locations/*/quotaAdjusterSettings}Z<\x12:/v1beta/{name=folders/*/locations/*/quotaAdjusterSettings}ZB\x12@/v1beta/{name=organizations/*/locations/*/quotaAdjusterSettings}\x1aN\xca\x41\x1a\x63loudquotas.googleapis.com\xd2\x41.https://www.googleapis.com/auth/cloud-platformB\xf1\x01\n!com.google.api.cloudquotas.v1betaB\x1aQuotaAdjusterSettingsProtoP\x01ZEcloud.google.com/go/cloudquotas/apiv1beta/cloudquotaspb;cloudquotaspb\xaa\x02\x1fGoogle.Cloud.CloudQuotas.V1Beta\xca\x02\x1fGoogle\\Cloud\\CloudQuotas\\V1beta\xea\x02\"Google::Cloud::CloudQuotas::V1betab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.api.cloudquotas.v1beta.quota_adjuster_settings_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n!com.google.api.cloudquotas.v1betaB\032QuotaAdjusterSettingsProtoP\001ZEcloud.google.com/go/cloudquotas/apiv1beta/cloudquotaspb;cloudquotaspb\252\002\037Google.Cloud.CloudQuotas.V1Beta\312\002\037Google\\Cloud\\CloudQuotas\\V1beta\352\002\"Google::Cloud::CloudQuotas::V1beta'
  _globals['_GETQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['name']._serialized_options = b'\340A\002\372A2\n0cloudquotas.googleapis.com/QuotaAdjusterSettings'
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['quota_adjuster_settings']._loaded_options = None
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['quota_adjuster_settings']._serialized_options = b'\340A\002'
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['update_mask']._serialized_options = b'\340A\001'
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['validate_only']._loaded_options = None
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST'].fields_by_name['validate_only']._serialized_options = b'\340A\001'
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['name']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['name']._serialized_options = b'\340A\010'
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['enablement']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['enablement']._serialized_options = b'\340A\001'
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['update_time']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['update_time']._serialized_options = b'\340A\003'
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['etag']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['etag']._serialized_options = b'\340A\001'
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['inherited']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['inherited']._serialized_options = b'\340A\001'
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['inherited_from']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS'].fields_by_name['inherited_from']._serialized_options = b'\340A\003'
  _globals['_QUOTAADJUSTERSETTINGS']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGS']._serialized_options = b'\352A\250\002\n0cloudquotas.googleapis.com/QuotaAdjusterSettings\022=projects/{project}/locations/{location}/quotaAdjusterSettings\022Gorganizations/{organization}/locations/{location}/quotaAdjusterSettings\022;folders/{folder}/locations/{location}/quotaAdjusterSettings*\025quotaAdjusterSettings2\025quotaAdjusterSettingsR\001\001'
  _globals['_QUOTAADJUSTERSETTINGSMANAGER']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGSMANAGER']._serialized_options = b'\312A\032cloudquotas.googleapis.com\322A.https://www.googleapis.com/auth/cloud-platform'
  _globals['_QUOTAADJUSTERSETTINGSMANAGER'].methods_by_name['UpdateQuotaAdjusterSettings']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGSMANAGER'].methods_by_name['UpdateQuotaAdjusterSettings']._serialized_options = b'\332A#quota_adjuster_settings,update_mask\202\323\344\223\002\322\0022S/v1beta/{quota_adjuster_settings.name=projects/*/locations/*/quotaAdjusterSettings}:\027quota_adjuster_settingsZm2R/v1beta/{quota_adjuster_settings.name=folders/*/locations/*/quotaAdjusterSettings}:\027quota_adjuster_settingsZs2X/v1beta/{quota_adjuster_settings.name=organizations/*/locations/*/quotaAdjusterSettings}:\027quota_adjuster_settings'
  _globals['_QUOTAADJUSTERSETTINGSMANAGER'].methods_by_name['GetQuotaAdjusterSettings']._loaded_options = None
  _globals['_QUOTAADJUSTERSETTINGSMANAGER'].methods_by_name['GetQuotaAdjusterSettings']._serialized_options = b'\332A\004name\202\323\344\223\002\277\001\022;/v1beta/{name=projects/*/locations/*/quotaAdjusterSettings}Z<\022:/v1beta/{name=folders/*/locations/*/quotaAdjusterSettings}ZB\022@/v1beta/{name=organizations/*/locations/*/quotaAdjusterSettings}'
  _globals['_GETQUOTAADJUSTERSETTINGSREQUEST']._serialized_start=276
  _globals['_GETQUOTAADJUSTERSETTINGSREQUEST']._serialized_end=381
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST']._serialized_start=384
  _globals['_UPDATEQUOTAADJUSTERSETTINGSREQUEST']._serialized_end=594
  _globals['_QUOTAADJUSTERSETTINGS']._serialized_start=597
  _globals['_QUOTAADJUSTERSETTINGS']._serialized_end=1227
  _globals['_QUOTAADJUSTERSETTINGS_ENABLEMENT']._serialized_start=857
  _globals['_QUOTAADJUSTERSETTINGS_ENABLEMENT']._serialized_end=924
  _globals['_QUOTAADJUSTERSETTINGSMANAGER']._serialized_start=1230
  _globals['_QUOTAADJUSTERSETTINGSMANAGER']._serialized_end=2234
# @@protoc_insertion_point(module_scope)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc_channelz.v1.channelz."""

import asyncio
import ipaddress
import logging
import unittest

import grpc
from grpc.experimental import aio
from grpc_channelz.v1 import channelz
from grpc_channelz.v1 import channelz_pb2
from grpc_channelz.v1 import channelz_pb2_grpc

from tests.unit.framework.common import test_constants
from tests_aio.unit._test_base import AioTestBase

_SUCCESSFUL_UNARY_UNARY = "/test/SuccessfulUnaryUnary"
_FAILED_UNARY_UNARY = "/test/FailedUnaryUnary"
_SUCCESSFUL_STREAM_STREAM = "/test/SuccessfulStreamStream"

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x01\x01\x01"

_DISABLE_REUSE_PORT = (("grpc.so_reuseport", 0),)
_ENABLE_CHANNELZ = (("grpc.enable_channelz", 1),)
_DISABLE_CHANNELZ = (("grpc.enable_channelz", 0),)

_LARGE_UNASSIGNED_ID = 10000


async def _successful_unary_unary(request, servicer_context):
    return _RESPONSE


async def _failed_unary_unary(request, servicer_context):
    servicer_context.set_code(grpc.StatusCode.INTERNAL)
    servicer_context.set_details("Channelz Test Intended Failure")


async def _successful_stream_stream(request_iterator, servicer_context):
    async for _ in request_iterator:
        yield _RESPONSE


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == _SUCCESSFUL_UNARY_UNARY:
            return grpc.unary_unary_rpc_method_handler(_successful_unary_unary)
        elif handler_call_details.method == _FAILED_UNARY_UNARY:
            return grpc.unary_unary_rpc_method_handler(_failed_unary_unary)
        elif handler_call_details.method == _SUCCESSFUL_STREAM_STREAM:
            return grpc.stream_stream_rpc_method_handler(
                _successful_stream_stream
            )
        else:
            return None


class _ChannelServerPair:
    def __init__(self):
        self.address = ""
        self.server = None
        self.channel = None
        self.server_ref_id = None
        self.channel_ref_id = None

    async def start(self):
        # Server will enable channelz service
        self.server = aio.server(options=_DISABLE_REUSE_PORT + _ENABLE_CHANNELZ)
        port = self.server.add_insecure_port("[::]:0")
        self.address = "localhost:%d" % port
        self.server.add_generic_rpc_handlers((_GenericHandler(),))
        await self.server.start()

        # Channel will enable channelz service...
        self.channel = aio.insecure_channel(
            self.address, options=_ENABLE_CHANNELZ
        )

    async def bind_channelz(self, channelz_stub):
        resp = await channelz_stub.GetTopChannels(
            channelz_pb2.GetTopChannelsRequest(start_channel_id=0)
        )
        for channel in resp.channel:
            if channel.data.target == "dns:///" + self.address:
                self.channel_ref_id = channel.ref.channel_id

        resp = await channelz_stub.GetServers(
            channelz_pb2.GetServersRequest(start_server_id=0)
        )
        self.server_ref_id = resp.server[-1].ref.server_id

    async def stop(self):
        await self.channel.close()
        await self.server.stop(None)


async def _create_channel_server_pairs(n, channelz_stub=None):
    """Create channel-server pairs."""
    pairs = [_ChannelServerPair() for i in range(n)]
    for pair in pairs:
        await pair.start()
        if channelz_stub:
            await pair.bind_channelz(channelz_stub)
    return pairs


async def _destroy_channel_server_pairs(pairs):
    for pair in pairs:
        await pair.stop()


class ChannelzServicerTest(AioTestBase):
    async def setUp(self):
        # This server is for Channelz info fetching only
        # It self should not enable Channelz
        self._server = aio.server(
            options=_DISABLE_REUSE_PORT + _DISABLE_CHANNELZ
        )
        port = self._server.add_insecure_port("[::]:0")
        channelz.add_channelz_servicer(self._server)
        await self._server.start()

        # This channel is used to fetch Channelz info only
        # Channelz should not be enabled
        self._channel = aio.insecure_channel(
            "localhost:%d" % port, options=_DISABLE_CHANNELZ
        )
        self._channelz_stub = channelz_pb2_grpc.ChannelzStub(self._channel)

    async def tearDown(self):
        await self._channel.close()
        await self._server.stop(None)

    async def _get_server_by_ref_id(self, ref_id):
        """Server id may not be consecutive"""
        resp = await self._channelz_stub.GetServers(
            channelz_pb2.GetServersRequest(start_server_id=ref_id)
        )
        self.assertEqual(ref_id, resp.server[0].ref.server_id)
        return resp.server[0]

    async def _send_successful_unary_unary(self, pair):
        call = pair.channel.unary_unary(_SUCCESSFUL_UNARY_UNARY)(_REQUEST)
        self.assertEqual(grpc.StatusCode.OK, await call.code())

    async def _send_failed_unary_unary(self, pair):
        try:
            await pair.channel.unary_unary(_FAILED_UNARY_UNARY)(_REQUEST)
        except grpc.RpcError:
            return
        else:
            self.fail("This call supposed to fail")

    async def _send_successful_stream_stream(self, pair):
        call = pair.channel.stream_stream(_SUCCESSFUL_STREAM_STREAM)(
            iter([_REQUEST] * test_constants.STREAM_LENGTH)
        )
        cnt = 0
        async for _ in call:
            cnt += 1
        self.assertEqual(cnt, test_constants.STREAM_LENGTH)

    async def test_get_top_channels_high_start_id(self):
        pairs = await _create_channel_server_pairs(1)

        resp = await self._channelz_stub.GetTopChannels(
            channelz_pb2.GetTopChannelsRequest(
                start_channel_id=_LARGE_UNASSIGNED_ID
            )
        )
        self.assertEqual(len(resp.channel), 0)
        self.assertEqual(resp.end, True)

        await _destroy_channel_server_pairs(pairs)

    async def test_successful_request(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)

        await self._send_successful_unary_unary(pairs[0])
        resp = await self._channelz_stub.GetChannel(
            channelz_pb2.GetChannelRequest(channel_id=pairs[0].channel_ref_id)
        )

        self.assertEqual(resp.channel.data.calls_started, 1)
        self.assertEqual(resp.channel.data.calls_succeeded, 1)
        self.assertEqual(resp.channel.data.calls_failed, 0)

        await _destroy_channel_server_pairs(pairs)

    async def test_failed_request(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)

        await self._send_failed_unary_unary(pairs[0])
        resp = await self._channelz_stub.GetChannel(
            channelz_pb2.GetChannelRequest(channel_id=pairs[0].channel_ref_id)
        )
        self.assertEqual(resp.channel.data.calls_started, 1)
        self.assertEqual(resp.channel.data.calls_succeeded, 0)
        self.assertEqual(resp.channel.data.calls_failed, 1)

        await _destroy_channel_server_pairs(pairs)

    async def test_many_requests(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)

        k_success = 7
        k_failed = 9
        for i in range(k_success):
            await self._send_successful_unary_unary(pairs[0])
        for i in range(k_failed):
            await self._send_failed_unary_unary(pairs[0])
        resp = await self._channelz_stub.GetChannel(
            channelz_pb2.GetChannelRequest(channel_id=pairs[0].channel_ref_id)
        )
        self.assertEqual(resp.channel.data.calls_started, k_success + k_failed)
        self.assertEqual(resp.channel.data.calls_succeeded, k_success)
        self.assertEqual(resp.channel.data.calls_failed, k_failed)

        await _destroy_channel_server_pairs(pairs)

    async def test_many_subchannels(self):
        k_channels = 4
        pairs = await _create_channel_server_pairs(
            k_channels, self._channelz_stub
        )
        k_success = 17
        k_failed = 19
        for i in range(k_success):
            await self._send_successful_unary_unary(pairs[0])
            await self._send_successful_unary_unary(pairs[2])
        for i in range(k_failed):
            await self._send_failed_unary_unary(pairs[1])
            await self._send_failed_unary_unary(pairs[2])

        for i in range(k_channels):
            gc_resp = await self._channelz_stub.GetChannel(
                channelz_pb2.GetChannelRequest(
                    channel_id=pairs[i].channel_ref_id
                )
            )
            # If no call performed in the channel, there shouldn't be any subchannel
            if gc_resp.channel.data.calls_started == 0:
                self.assertEqual(len(gc_resp.channel.subchannel_ref), 0)
                continue

            # Otherwise, the subchannel should exist
            self.assertGreater(len(gc_resp.channel.subchannel_ref), 0)
            gsc_resp = await self._channelz_stub.GetSubchannel(
                channelz_pb2.GetSubchannelRequest(
                    subchannel_id=gc_resp.channel.subchannel_ref[
                        0
                    ].subchannel_id
                )
            )
            self.assertEqual(
                gc_resp.channel.data.calls_started,
                gsc_resp.subchannel.data.calls_started,
            )
            self.assertEqual(
                gc_resp.channel.data.calls_succeeded,
                gsc_resp.subchannel.data.calls_succeeded,
            )
            self.assertEqual(
                gc_resp.channel.data.calls_failed,
                gsc_resp.subchannel.data.calls_failed,
            )

        await _destroy_channel_server_pairs(pairs)

    async def test_server_call(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)

        k_success = 23
        k_failed = 29
        for i in range(k_success):
            await self._send_successful_unary_unary(pairs[0])
        for i in range(k_failed):
            await self._send_failed_unary_unary(pairs[0])

        resp = await self._get_server_by_ref_id(pairs[0].server_ref_id)
        self.assertEqual(resp.data.calls_started, k_success + k_failed)
        self.assertEqual(resp.data.calls_succeeded, k_success)
        self.assertEqual(resp.data.calls_failed, k_failed)

        await _destroy_channel_server_pairs(pairs)

    async def test_many_subchannels_and_sockets(self):
        k_channels = 4
        pairs = await _create_channel_server_pairs(
            k_channels, self._channelz_stub
        )
        k_success = 3
        k_failed = 5
        for i in range(k_success):
            await self._send_successful_unary_unary(pairs[0])
            await self._send_successful_unary_unary(pairs[2])
        for i in range(k_failed):
            await self._send_failed_unary_unary(pairs[1])
            await self._send_failed_unary_unary(pairs[2])

        for i in range(k_channels):
            gc_resp = await self._channelz_stub.GetChannel(
                channelz_pb2.GetChannelRequest(
                    channel_id=pairs[i].channel_ref_id
                )
            )

            # If no call performed in the channel, there shouldn't be any subchannel
            if gc_resp.channel.data.calls_started == 0:
                self.assertEqual(len(gc_resp.channel.subchannel_ref), 0)
                continue

            # Otherwise, the subchannel should exist
            self.assertGreater(len(gc_resp.channel.subchannel_ref), 0)
            gsc_resp = await self._channelz_stub.GetSubchannel(
                channelz_pb2.GetSubchannelRequest(
                    subchannel_id=gc_resp.channel.subchannel_ref[
                        0
                    ].subchannel_id
                )
            )
            self.assertEqual(len(gsc_resp.subchannel.socket_ref), 1)

            gs_resp = await self._channelz_stub.GetSocket(
                channelz_pb2.GetSocketRequest(
                    socket_id=gsc_resp.subchannel.socket_ref[0].socket_id
                )
            )
            self.assertEqual(
                gsc_resp.subchannel.data.calls_started,
                gs_resp.socket.data.streams_started,
            )
            self.assertEqual(0, gs_resp.socket.data.streams_failed)
            # Calls started == messages sent, only valid for unary calls
            self.assertEqual(
                gsc_resp.subchannel.data.calls_started,
                gs_resp.socket.data.messages_sent,
            )

        await _destroy_channel_server_pairs(pairs)

    async def test_streaming_rpc(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)
        # In C++, the argument for _send_successful_stream_stream is message length.
        # Here the argument is still channel idx, to be consistent with the other two.
        await self._send_successful_stream_stream(pairs[0])

        gc_resp = await self._channelz_stub.GetChannel(
            channelz_pb2.GetChannelRequest(channel_id=pairs[0].channel_ref_id)
        )
        self.assertEqual(gc_resp.channel.data.calls_started, 1)
        self.assertEqual(gc_resp.channel.data.calls_succeeded, 1)
        self.assertEqual(gc_resp.channel.data.calls_failed, 0)
        # Subchannel exists
        self.assertGreater(len(gc_resp.channel.subchannel_ref), 0)

        while True:
            gsc_resp = await self._channelz_stub.GetSubchannel(
                channelz_pb2.GetSubchannelRequest(
                    subchannel_id=gc_resp.channel.subchannel_ref[
                        0
                    ].subchannel_id
                )
            )
            if (
                gsc_resp.subchannel.data.calls_started
                == gsc_resp.subchannel.data.calls_succeeded
                + gsc_resp.subchannel.data.calls_failed
            ):
                break
        self.assertEqual(gsc_resp.subchannel.data.calls_started, 1)
        self.assertEqual(gsc_resp.subchannel.data.calls_failed, 0)
        self.assertEqual(gsc_resp.subchannel.data.calls_succeeded, 1)
        # Socket exists
        self.assertEqual(len(gsc_resp.subchannel.socket_ref), 1)

        while True:
            gs_resp = await self._channelz_stub.GetSocket(
                channelz_pb2.GetSocketRequest(
                    socket_id=gsc_resp.subchannel.socket_ref[0].socket_id
                )
            )
            if (
                gs_resp.socket.data.streams_started
                == gs_resp.socket.data.streams_succeeded
                + gs_resp.socket.data.streams_failed
            ):
                break
        self.assertEqual(gs_resp.socket.data.streams_started, 1)
        self.assertEqual(gs_resp.socket.data.streams_failed, 0)
        self.assertEqual(gs_resp.socket.data.streams_succeeded, 1)
        self.assertEqual(
            gs_resp.socket.data.messages_sent, test_constants.STREAM_LENGTH
        )
        self.assertEqual(
            gs_resp.socket.data.messages_received, test_constants.STREAM_LENGTH
        )

        await _destroy_channel_server_pairs(pairs)

    async def test_server_sockets(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)

        await self._send_successful_unary_unary(pairs[0])
        await self._send_failed_unary_unary(pairs[0])

        resp = await self._get_server_by_ref_id(pairs[0].server_ref_id)
        self.assertEqual(resp.data.calls_started, 2)
        self.assertEqual(resp.data.calls_succeeded, 1)
        self.assertEqual(resp.data.calls_failed, 1)

        gss_resp = await self._channelz_stub.GetServerSockets(
            channelz_pb2.GetServerSocketsRequest(
                server_id=resp.ref.server_id, start_socket_id=0
            )
        )

        self.assertEqual(len(gss_resp.socket_ref), 1)

        gs_resp: channelz_pb2.GetSocketResponse = (
            await self._channelz_stub.GetSocket(
                channelz_pb2.GetSocketRequest(
                    socket_id=gss_resp.socket_ref[0].socket_id
                )
            )
        )

        # Verify the client socket contains valid TCP/IP addresses.
        client_socket: channelz_pb2.Socket = gs_resp.socket

        # Verify server's local address is a valid TCP/IP addresses.
        local_address: channelz_pb2.Address = client_socket.local
        self.assertEqual(
            local_address.WhichOneof("address"),
            "tcpip_address",
            msg="Expected the socket to contain local TCP/IP address",
        )
        address_str = self.tcpip_address_to_str(local_address.tcpip_address)
        self.assertNotIn(
            address_str,
            ("", ":"),
            msg="Server address string must not be empty",
        )

        # Verify server's remote address (to the client) is
        # a valid TCP/IP addresses.
        remote_address: channelz_pb2.Address = client_socket.remote
        self.assertEqual(
            remote_address.WhichOneof("address"),
            "tcpip_address",
            msg="Expected the socket to contain remote TCP/IP address",
        )
        address_str = self.tcpip_address_to_str(remote_address.tcpip_address)
        self.assertNotIn(
            address_str,
            ("", ":"),
            msg="Client address string must not be empty",
        )

        await _destroy_channel_server_pairs(pairs)

    async def test_server_listen_sockets(self):
        pairs = await _create_channel_server_pairs(1, self._channelz_stub)

        resp = await self._get_server_by_ref_id(pairs[0].server_ref_id)
        self.assertEqual(len(resp.listen_socket), 1)

        gs_resp: channelz_pb2.GetSocketResponse = (
            await self._channelz_stub.GetSocket(
                channelz_pb2.GetSocketRequest(
                    socket_id=resp.listen_socket[0].socket_id
                )
            )
        )

        # Verify the listen socket contains a valid TCP/IP address.
        listen_address: channelz_pb2.Address = gs_resp.socket.local
        self.assertEqual(
            listen_address.WhichOneof("address"),
            "tcpip_address",
            msg="Expected the server listen socket to contain a TCP/IP address",
        )
        address_str = self.tcpip_address_to_str(listen_address.tcpip_address)
        self.assertNotIn(
            address_str, ("", ":"), msg="Address string must not be empty"
        )

        await _destroy_channel_server_pairs(pairs)

    @staticmethod
    def is_ipv4(tcpip_address: channelz_pb2.Address.TcpIpAddress):
        # According to proto, tcpip_address.ip_address is either IPv4 or IPv6.
        # Correspondingly, it's either 4 bytes or 16 bytes in length.
        return len(tcpip_address.ip_address) == 4

    @classmethod
    def tcpip_address_to_str(
        cls, tcpip_address: channelz_pb2.Address.TcpIpAddress
    ):
        # This will throw AddressValueError when the address is invalid.
        if cls.is_ipv4(tcpip_address):
            ip = ipaddress.IPv4Address(tcpip_address.ip_address)
        else:
            ip = ipaddress.IPv6Address(tcpip_address.ip_address)
        return f"{ip}:{tcpip_address.port}"

    async def test_invalid_query_get_server(self):
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channelz_stub.GetServer(
                channelz_pb2.GetServerRequest(server_id=_LARGE_UNASSIGNED_ID)
            )
        self.assertEqual(
            grpc.StatusCode.NOT_FOUND, exception_context.exception.code()
        )

    async def test_invalid_query_get_channel(self):
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channelz_stub.GetChannel(
                channelz_pb2.GetChannelRequest(channel_id=_LARGE_UNASSIGNED_ID)
            )
        self.assertEqual(
            grpc.StatusCode.NOT_FOUND, exception_context.exception.code()
        )

    async def test_invalid_query_get_subchannel(self):
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channelz_stub.GetSubchannel(
                channelz_pb2.GetSubchannelRequest(
                    subchannel_id=_LARGE_UNASSIGNED_ID
                )
            )
        self.assertEqual(
            grpc.StatusCode.NOT_FOUND, exception_context.exception.code()
        )

    async def test_invalid_query_get_socket(self):
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channelz_stub.GetSocket(
                channelz_pb2.GetSocketRequest(socket_id=_LARGE_UNASSIGNED_ID)
            )
        self.assertEqual(
            grpc.StatusCode.NOT_FOUND, exception_context.exception.code()
        )

    async def test_invalid_query_get_server_sockets(self):
        with self.assertRaises(aio.AioRpcError) as exception_context:
            await self._channelz_stub.GetServerSockets(
                channelz_pb2.GetServerSocketsRequest(
                    server_id=_LARGE_UNASSIGNED_ID,
                    start_socket_id=0,
                )
            )
        self.assertEqual(
            grpc.StatusCode.NOT_FOUND, exception_context.exception.code()
        )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from concurrent import futures
from typing import Tuple

import grpc

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_UNARY_FILTERED = "UnaryUnaryFiltered"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"
STREAM_LENGTH = 5
TRIGGER_RPC_METADATA = ("control", "trigger_rpc")
TRIGGER_RPC_TO_NEW_SERVER_METADATA = ("to_new_server", "")


def handle_unary_unary(request, servicer_context):
    if TRIGGER_RPC_METADATA in servicer_context.invocation_metadata():
        for k, v in servicer_context.invocation_metadata():
            if "port" in k:
                unary_unary_call(port=int(v))
            if "to_new_server" in k:
                second_server = grpc.server(
                    futures.ThreadPoolExecutor(max_workers=10)
                )
                second_server.add_generic_rpc_handlers((_GenericHandler(),))
                second_server_port = second_server.add_insecure_port("[::]:0")
                second_server.start()
                unary_unary_call(port=second_server_port)
                second_server.stop(0)
    return _RESPONSE


def handle_unary_stream(request, servicer_context):
    for _ in range(STREAM_LENGTH):
        yield _RESPONSE


def handle_stream_unary(request_iterator, servicer_context):
    return _RESPONSE


def handle_stream_stream(request_iterator, servicer_context):
    for request in request_iterator:
        yield _RESPONSE


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, request_streaming, response_streaming):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            self.stream_stream = handle_stream_stream
        elif self.request_streaming:
            self.stream_unary = handle_stream_unary
        elif self.response_streaming:
            self.unary_stream = handle_unary_stream
        else:
            self.unary_unary = handle_unary_unary


class _GenericHandler(grpc.GenericRpcHandler):
    def service(self, handler_call_details):
        if handler_call_details.method == _UNARY_UNARY:
            return _MethodHandler(False, False)
        if handler_call_details.method == _UNARY_UNARY_FILTERED:
            return _MethodHandler(False, False)
        elif handler_call_details.method == _UNARY_STREAM:
            return _MethodHandler(False, True)
        elif handler_call_details.method == _STREAM_UNARY:
            return _MethodHandler(True, False)
        elif handler_call_details.method == _STREAM_STREAM:
            return _MethodHandler(True, True)
        else:
            return None


RPC_METHOD_HANDLERS = {
    _UNARY_UNARY_FILTERED: _MethodHandler(False, False),
    _UNARY_UNARY: _MethodHandler(False, False),
    _UNARY_STREAM: _MethodHandler(False, True),
    _STREAM_UNARY: _MethodHandler(True, False),
    _STREAM_STREAM: _MethodHandler(True, True),
}

REGISTERED_RPC_METHOD_HANDLERS = {
    _UNARY_UNARY_FILTERED: _MethodHandler(False, False),
    _UNARY_UNARY: _MethodHandler(False, False),
    _UNARY_STREAM: _MethodHandler(False, True),
    _STREAM_UNARY: _MethodHandler(True, False),
    _STREAM_STREAM: _MethodHandler(True, True),
}


def start_server(
    interceptors=None, register_method=True
) -> Tuple[grpc.Server, int]:
    if interceptors:
        server = grpc.server(
            futures.ThreadPoolExecutor(max_workers=10),
            interceptors=interceptors,
        )
    else:
        server = grpc.server(futures.ThreadPoolExecutor(max_workers=10))
    generic_handler = grpc.method_handlers_generic_handler(
        _SERVICE_NAME, RPC_METHOD_HANDLERS
    )
    server.add_generic_rpc_handlers((generic_handler,))
    if register_method:
        server.add_registered_method_handlers(
            _SERVICE_NAME, REGISTERED_RPC_METHOD_HANDLERS
        )
    port = server.add_insecure_port("[::]:0")
    server.start()
    return server, port


def unary_unary_call(port, metadata=None, registered_method=False):
    with grpc.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.unary_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
            _registered_method=registered_method,
        )
        if metadata:
            unused_response, call = multi_callable.with_call(
                _REQUEST, metadata=metadata
            )
        else:
            unused_response, call = multi_callable.with_call(_REQUEST)


def intercepted_unary_unary_call(port, interceptors, metadata=None):
    with grpc.insecure_channel(f"localhost:{port}") as channel:
        intercept_channel = grpc.intercept_channel(channel, interceptors)
        multi_callable = intercept_channel.unary_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY)
        )
        if metadata:
            unused_response, call = multi_callable.with_call(
                _REQUEST, metadata=metadata
            )
        else:
            unused_response, call = multi_callable.with_call(_REQUEST)


def unary_unary_filtered_call(port, metadata=None):
    with grpc.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.unary_unary(
            grpc._common.fully_qualified_method(
                _SERVICE_NAME, _UNARY_UNARY_FILTERED
            )
        )
        if metadata:
            unused_response, call = multi_callable.with_call(
                _REQUEST, metadata=metadata
            )
        else:
            unused_response, call = multi_callable.with_call(_REQUEST)


def unary_stream_call(port):
    with grpc.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.unary_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM)
        )
        call = multi_callable(_REQUEST)
        for _ in call:
            pass


def stream_unary_call(port):
    with grpc.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.stream_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY)
        )
        unused_response, call = multi_callable.with_call(
            iter([_REQUEST] * STREAM_LENGTH)
        )


def stream_stream_call(port):
    with grpc.insecure_channel(f"localhost:{port}") as channel:
        multi_callable = channel.stream_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM)
        )
        call = multi_callable(iter([_REQUEST] * STREAM_LENGTH))
        for _ in call:
            pass



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nmenvoy/extensions/http/ext_proc/response_processors/save_processing_response/v3/save_processing_response.proto\x12Nenvoy.extensions.http.ext_proc.response_processors.save_processing_response.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\xdd\x06\n\x16SaveProcessingResponse\x12 \n\x18\x66ilter_state_name_suffix\x18\x01 \x01(\t\x12\x90\x01\n\x14save_request_headers\x18\x02 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x91\x01\n\x15save_response_headers\x18\x03 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x91\x01\n\x15save_request_trailers\x18\x04 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x92\x01\n\x16save_response_trailers\x18\x05 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x12\x93\x01\n\x17save_immediate_response\x18\x06 \x01(\x0b\x32r.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.SaveProcessingResponse.SaveOptions\x1a;\n\x0bSaveOptions\x12\x15\n\rsave_response\x18\x01 \x01(\x08\x12\x15\n\rsave_on_error\x18\x02 \x01(\x08\x42\xa0\x02\n\\io.envoyproxy.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3B\x1bSaveProcessingResponseProtoP\x01Z\x90\x01github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3;save_processing_responsev3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3.save_processing_response_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\\io.envoyproxy.envoy.extensions.http.ext_proc.response_processors.save_processing_response.v3B\033SaveProcessingResponseProtoP\001Z\220\001github.com/envoyproxy/go-control-plane/envoy/extensions/http/ext_proc/response_processors/save_processing_response/v3;save_processing_responsev3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_SAVEPROCESSINGRESPONSE']._serialized_start=258
  _globals['_SAVEPROCESSINGRESPONSE']._serialized_end=1119
  _globals['_SAVEPROCESSINGRESPONSE_SAVEOPTIONS']._serialized_start=1060
  _globals['_SAVEPROCESSINGRESPONSE_SAVEOPTIONS']._serialized_end=1119
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nNenvoy/extensions/grpc_service/channel_credentials/xds/v3/xds_credentials.proto\x12\x38\x65nvoy.extensions.grpc_service.channel_credentials.xds.v3\x1a\x19google/protobuf/any.proto\x1a\x1dudpa/annotations/status.proto\"D\n\x0eXdsCredentials\x12\x32\n\x14\x66\x61llback_credentials\x18\x01 \x01(\x0b\x32\x14.google.protobuf.AnyB\xce\x01\nFio.envoyproxy.envoy.extensions.grpc_service.channel_credentials.xds.v3B\x13XdsCredentialsProtoP\x01Zegithub.com/envoyproxy/go-control-plane/envoy/extensions/grpc_service/channel_credentials/xds/v3;xdsv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.grpc_service.channel_credentials.xds.v3.xds_credentials_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nFio.envoyproxy.envoy.extensions.grpc_service.channel_credentials.xds.v3B\023XdsCredentialsProtoP\001Zegithub.com/envoyproxy/go-control-plane/envoy/extensions/grpc_service/channel_credentials/xds/v3;xdsv3\272\200\310\321\006\002\020\002'
  _globals['_XDSCREDENTIALS']._serialized_start=198
  _globals['_XDSCREDENTIALS']._serialized_end=266
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python2.7
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Generates the appropriate build.json data for all the proto files."""

import yaml
import collections
import os
import re
import sys


def update_deps(key, proto_filename, deps, deps_external, is_trans, visited):
    if not proto_filename in visited:
        visited.append(proto_filename)
        with open(proto_filename) as inp:
            for line in inp:
                imp = re.search(r'import "([^"]*)"', line)
                if not imp:
                    continue
                imp_proto = imp.group(1)
                # This indicates an external dependency, which we should handle
                # differently and not traverse recursively
                if imp_proto.startswith("google/"):
                    if key not in deps_external:
                        deps_external[key] = []
                    deps_external[key].append(imp_proto[:-6])
                    continue
                if imp_proto.startswith("xds/"):
                    if key not in deps_external:
                        deps_external[key] = []
                    deps_external[key].append(imp_proto[:-3])
                    continue
                if imp_proto.startswith("envoy/"):
                    if key not in deps_external:
                        deps_external[key] = []
                    deps_external[key].append(imp_proto[:-5])
                    continue
                # In case that the path is changed by copybara,
                # revert the change to avoid file error.
                if imp_proto.startswith("third_party/grpc"):
                    imp_proto = imp_proto[17:]
                if key not in deps:
                    deps[key] = []
                deps[key].append(imp_proto[:-6])
                if is_trans:
                    update_deps(
                        key, imp_proto, deps, deps_external, is_trans, visited
                    )


def main():
    proto_dir = os.path.abspath(os.path.dirname(sys.argv[0]))
    os.chdir(os.path.join(proto_dir, "../.."))

    deps = {}
    deps_trans = {}
    deps_external = {}
    deps_external_trans = {}
    for root, dirs, files in os.walk("src/proto"):
        for f in files:
            if f[-6:] != ".proto":
                continue
            look_at = os.path.join(root, f)
            deps_for = look_at[:-6]
            # First level deps
            update_deps(deps_for, look_at, deps, deps_external, False, [])
            # Transitive deps
            update_deps(
                deps_for, look_at, deps_trans, deps_external_trans, True, []
            )

    json = {
        "proto_deps": deps,
        "proto_transitive_deps": deps_trans,
        "proto_external_deps": deps_external,
        "proto_transitive_external_deps": deps_external_trans,
    }

    print(yaml.dump(json))


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import address_pb2 as envoy_dot_config_dot_core_dot_v3_dot_address__pb2
from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import grpc_service_pb2 as envoy_dot_config_dot_core_dot_v3_dot_grpc__service__pb2
from envoy.extensions.common.aws.v3 import credential_provider_pb2 as envoy_dot_extensions_dot_common_dot_aws_dot_v3_dot_credential__provider__pb2
from envoy.extensions.common.dynamic_forward_proxy.v3 import dns_cache_pb2 as envoy_dot_extensions_dot_common_dot_dynamic__forward__proxy_dot_v3_dot_dns__cache__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import sensitive_pb2 as udpa_dot_annotations_dot_sensitive__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAenvoy/extensions/filters/network/redis_proxy/v3/redis_proxy.proto\x12/envoy.extensions.filters.network.redis_proxy.v3\x1a\"envoy/config/core/v3/address.proto\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a\'envoy/config/core/v3/grpc_service.proto\x1a\x38\x65nvoy/extensions/common/aws/v3/credential_provider.proto\x1a@envoy/extensions/common/dynamic_forward_proxy/v3/dns_cache.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xcf\x18\n\nRedisProxy\x12\x1c\n\x0bstat_prefix\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12h\n\x08settings\x18\x03 \x01(\x0b\x32L.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.ConnPoolSettingsB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x19\n\x11latency_in_micros\x18\x04 \x01(\x08\x12_\n\rprefix_routes\x18\x05 \x01(\x0b\x32H.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes\x12U\n\x18\x64ownstream_auth_password\x18\x06 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x11\x18\x01\xb8\xb7\x8b\xa4\x02\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12K\n\x19\x64ownstream_auth_passwords\x18\t \x03(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12V\n\x06\x66\x61ults\x18\x08 \x03(\x0b\x32\x46.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.RedisFault\x12J\n\x18\x64ownstream_auth_username\x18\x07 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12j\n\x16\x65xternal_auth_provider\x18\n \x01(\x0b\x32J.envoy.extensions.filters.network.redis_proxy.v3.RedisExternalAuthProvider\x12\x17\n\x0f\x63ustom_commands\x18\x0b \x03(\t\x1a\xdb\x06\n\x10\x43onnPoolSettings\x12\x37\n\nop_timeout\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02\x08\x01\x12\x1a\n\x12\x65nable_hashtagging\x18\x02 \x01(\x08\x12\x1a\n\x12\x65nable_redirection\x18\x03 \x01(\x08\x12Z\n\x10\x64ns_cache_config\x18\t \x01(\x0b\x32@.envoy.extensions.common.dynamic_forward_proxy.v3.DnsCacheConfig\x12$\n\x1cmax_buffer_size_before_flush\x18\x04 \x01(\r\x12\x37\n\x14\x62uffer_flush_timeout\x18\x05 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x46\n max_upstream_unknown_connections\x18\x06 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x1c\n\x14\x65nable_command_stats\x18\x08 \x01(\x08\x12v\n\x0bread_policy\x18\x07 \x01(\x0e\x32W.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.ConnPoolSettings.ReadPolicyB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12n\n\x15\x63onnection_rate_limit\x18\n \x01(\x0b\x32O.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.ConnectionRateLimit\"~\n\nReadPolicy\x12\x1b\n\x06MASTER\x10\x00\x1a\x0f\xf2\x98\xfe\x8f\x05\t\n\x07PRIMARY\x12)\n\rPREFER_MASTER\x10\x01\x1a\x16\xf2\x98\xfe\x8f\x05\x10\n\x0ePREFER_PRIMARY\x12\x0b\n\x07REPLICA\x10\x02\x12\x12\n\x0ePREFER_REPLICA\x10\x03\x12\x07\n\x03\x41NY\x10\x04:M\x9a\xc5\x88\x1eH\nFenvoy.config.filter.network.redis_proxy.v2.RedisProxy.ConnPoolSettings\x1a\xc4\x08\n\x0cPrefixRoutes\x12^\n\x06routes\x18\x01 \x03(\x0b\x32N.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route\x12\x18\n\x10\x63\x61se_insensitive\x18\x02 \x01(\x08\x12g\n\x0f\x63\x61tch_all_route\x18\x04 \x01(\x0b\x32N.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route\x1a\xec\x05\n\x05Route\x12\x18\n\x06prefix\x18\x01 \x01(\tB\x08\xfa\x42\x05r\x03(\xe8\x07\x12\x15\n\rremove_prefix\x18\x02 \x01(\x08\x12\x18\n\x07\x63luster\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x81\x01\n\x15request_mirror_policy\x18\x04 \x03(\x0b\x32\x62.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route.RequestMirrorPolicy\x12\x15\n\rkey_formatter\x18\x05 \x01(\t\x12}\n\x13read_command_policy\x18\x06 \x01(\x0b\x32`.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.PrefixRoutes.Route.ReadCommandPolicy\x1a\xfd\x01\n\x13RequestMirrorPolicy\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12H\n\x10runtime_fraction\x18\x02 \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercent\x12\x1d\n\x15\x65xclude_read_commands\x18\x03 \x01(\x08:c\x9a\xc5\x88\x1e^\n\\envoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route.RequestMirrorPolicy\x1a-\n\x11ReadCommandPolicy\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:O\x9a\xc5\x88\x1eJ\nHenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route:I\x9a\xc5\x88\x1e\x44\nBenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutesJ\x04\x08\x03\x10\x04R\x11\x63\x61tch_all_cluster\x1a\xb6\x02\n\nRedisFault\x12s\n\nfault_type\x18\x01 \x01(\x0e\x32U.envoy.extensions.filters.network.redis_proxy.v3.RedisProxy.RedisFault.RedisFaultTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12O\n\rfault_enabled\x18\x02 \x01(\x0b\x32..envoy.config.core.v3.RuntimeFractionalPercentB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12(\n\x05\x64\x65lay\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x10\n\x08\x63ommands\x18\x04 \x03(\t\"&\n\x0eRedisFaultType\x12\t\n\x05\x44\x45LAY\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x1a\x45\n\x13\x43onnectionRateLimit\x12.\n\x1d\x63onnection_rate_limit_per_sec\x18\x01 \x01(\rB\x07\xfa\x42\x04*\x02 \x00:<\x9a\xc5\x88\x1e\x37\n5envoy.config.filter.network.redis_proxy.v2.RedisProxyJ\x04\x08\x02\x10\x03R\x07\x63luster\"\xd2\x04\n\x14RedisProtocolOptions\x12?\n\rauth_password\x18\x01 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12?\n\rauth_username\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12H\n\x07\x61ws_iam\x18\x03 \x01(\x0b\x32\x37.envoy.extensions.filters.network.redis_proxy.v3.AwsIam\x12\x65\n\x0b\x63redentials\x18\x04 \x03(\x0b\x32P.envoy.extensions.filters.network.redis_proxy.v3.RedisProtocolOptions.Credential\x1a\xbe\x01\n\nCredential\x12.\n\x07\x61\x64\x64ress\x18\x01 \x01(\x0b\x32\x1d.envoy.config.core.v3.Address\x12?\n\rauth_password\x18\x02 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01\x12?\n\rauth_username\x18\x03 \x01(\x0b\x32 .envoy.config.core.v3.DataSourceB\x06\xb8\xb7\x8b\xa4\x02\x01:F\x9a\xc5\x88\x1e\x41\n?envoy.config.filter.network.redis_proxy.v2.RedisProtocolOptions\"\xe2\x01\n\x06\x41wsIam\x12R\n\x13\x63redential_provider\x18\x01 \x01(\x0b\x32\x35.envoy.extensions.common.aws.v3.AwsCredentialProvider\x12\x1b\n\ncache_name\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x14\n\x0cservice_name\x18\x03 \x01(\t\x12\x0e\n\x06region\x18\x04 \x01(\t\x12\x41\n\x0f\x65xpiration_time\x18\x05 \x01(\x0b\x32\x19.google.protobuf.DurationB\r\xfa\x42\n\xaa\x01\x07\"\x03\x08\x84\x07\x32\x00\"t\n\x19RedisExternalAuthProvider\x12\x37\n\x0cgrpc_service\x18\x01 \x01(\x0b\x32!.envoy.config.core.v3.GrpcService\x12\x1e\n\x16\x65nable_auth_expiration\x18\x02 \x01(\x08\x42\xc0\x01\n=io.envoyproxy.envoy.extensions.filters.network.redis_proxy.v3B\x0fRedisProxyProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/redis_proxy/v3;redis_proxyv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.redis_proxy.v3.redis_proxy_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.filters.network.redis_proxy.v3B\017RedisProxyProtoP\001Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/redis_proxy/v3;redis_proxyv3\272\200\310\321\006\002\020\002'
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["MASTER"]._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["MASTER"]._serialized_options = b'\362\230\376\217\005\t\n\007PRIMARY'
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["PREFER_MASTER"]._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY'].values_by_name["PREFER_MASTER"]._serialized_options = b'\362\230\376\217\005\020\n\016PREFER_PRIMARY'
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['op_timeout']._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['op_timeout']._serialized_options = b'\372B\005\252\001\002\010\001'
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['read_policy']._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS'].fields_by_name['read_policy']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._loaded_options = None
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._serialized_options = b'\232\305\210\036H\nFenvoy.config.filter.network.redis_proxy.v2.RedisProxy.ConnPoolSettings'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY'].fields_by_name['cluster']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._serialized_options = b'\232\305\210\036^\n\\envoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route.RequestMirrorPolicy'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY'].fields_by_name['cluster']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['prefix']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['prefix']._serialized_options = b'\372B\005r\003(\350\007'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['cluster']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._serialized_options = b'\232\305\210\036J\nHenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes.Route'
  _globals['_REDISPROXY_PREFIXROUTES']._loaded_options = None
  _globals['_REDISPROXY_PREFIXROUTES']._serialized_options = b'\232\305\210\036D\nBenvoy.config.filter.network.redis_proxy.v2.RedisProxy.PrefixRoutes'
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_type']._loaded_options = None
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_enabled']._loaded_options = None
  _globals['_REDISPROXY_REDISFAULT'].fields_by_name['fault_enabled']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_REDISPROXY_CONNECTIONRATELIMIT'].fields_by_name['connection_rate_limit_per_sec']._loaded_options = None
  _globals['_REDISPROXY_CONNECTIONRATELIMIT'].fields_by_name['connection_rate_limit_per_sec']._serialized_options = b'\372B\004*\002 \000'
  _globals['_REDISPROXY'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_REDISPROXY'].fields_by_name['settings']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['settings']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_password']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_password']._serialized_options = b'\030\001\270\267\213\244\002\001\222\307\206\330\004\0033.0'
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_passwords']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_passwords']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_username']._loaded_options = None
  _globals['_REDISPROXY'].fields_by_name['downstream_auth_username']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROXY']._loaded_options = None
  _globals['_REDISPROXY']._serialized_options = b'\232\305\210\0367\n5envoy.config.filter.network.redis_proxy.v2.RedisProxy'
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_password']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_password']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_username']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL'].fields_by_name['auth_username']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_password']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_password']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_username']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS'].fields_by_name['auth_username']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_REDISPROTOCOLOPTIONS']._loaded_options = None
  _globals['_REDISPROTOCOLOPTIONS']._serialized_options = b'\232\305\210\036A\n?envoy.config.filter.network.redis_proxy.v2.RedisProtocolOptions'
  _globals['_AWSIAM'].fields_by_name['cache_name']._loaded_options = None
  _globals['_AWSIAM'].fields_by_name['cache_name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_AWSIAM'].fields_by_name['expiration_time']._loaded_options = None
  _globals['_AWSIAM'].fields_by_name['expiration_time']._serialized_options = b'\372B\n\252\001\007\"\003\010\204\0072\000'
  _globals['_REDISPROXY']._serialized_start=611
  _globals['_REDISPROXY']._serialized_end=3762
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._serialized_start=1347
  _globals['_REDISPROXY_CONNPOOLSETTINGS']._serialized_end=2206
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY']._serialized_start=2001
  _globals['_REDISPROXY_CONNPOOLSETTINGS_READPOLICY']._serialized_end=2127
  _globals['_REDISPROXY_PREFIXROUTES']._serialized_start=2209
  _globals['_REDISPROXY_PREFIXROUTES']._serialized_end=3301
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._serialized_start=2453
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE']._serialized_end=3201
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._serialized_start=2820
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_REQUESTMIRRORPOLICY']._serialized_end=3073
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY']._serialized_start=3075
  _globals['_REDISPROXY_PREFIXROUTES_ROUTE_READCOMMANDPOLICY']._serialized_end=3120
  _globals['_REDISPROXY_REDISFAULT']._serialized_start=3304
  _globals['_REDISPROXY_REDISFAULT']._serialized_end=3614
  _globals['_REDISPROXY_REDISFAULT_REDISFAULTTYPE']._serialized_start=3576
  _globals['_REDISPROXY_REDISFAULT_REDISFAULTTYPE']._serialized_end=3614
  _globals['_REDISPROXY_CONNECTIONRATELIMIT']._serialized_start=3616
  _globals['_REDISPROXY_CONNECTIONRATELIMIT']._serialized_end=3685
  _globals['_REDISPROTOCOLOPTIONS']._serialized_start=3765
  _globals['_REDISPROTOCOLOPTIONS']._serialized_end=4359
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL']._serialized_start=4097
  _globals['_REDISPROTOCOLOPTIONS_CREDENTIAL']._serialized_end=4287
  _globals['_AWSIAM']._serialized_start=4362
  _globals['_AWSIAM']._serialized_end=4588
  _globals['_REDISEXTERNALAUTHPROVIDER']._serialized_start=4590
  _globals['_REDISEXTERNALAUTHPROVIDER']._serialized_end=4706
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/compressor/v3/compressor.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/compressor/v3/compressor.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n<envoy/extensions/filters/http/compressor/v3/compressor.proto\x12+envoy.extensions.filters.http.compressor.v3\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a$envoy/config/core/v3/extension.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\xf3\t\n\nCompressor\x12\x41\n\x0e\x63ontent_length\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12!\n\x0c\x63ontent_type\x18\x02 \x03(\tB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12+\n\x16\x64isable_on_etag_header\x18\x03 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x32\n\x1dremove_accept_encoding_header\x18\x04 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12N\n\x0fruntime_enabled\x18\x05 \x01(\x0b\x32(.envoy.config.core.v3.RuntimeFeatureFlagB\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12P\n\x12\x63ompressor_library\x18\x06 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfigB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12p\n\x18request_direction_config\x18\x07 \x01(\x0b\x32N.envoy.extensions.filters.http.compressor.v3.Compressor.RequestDirectionConfig\x12r\n\x19response_direction_config\x18\x08 \x01(\x0b\x32O.envoy.extensions.filters.http.compressor.v3.Compressor.ResponseDirectionConfig\x12\x14\n\x0c\x63hoose_first\x18\t \x01(\x08\x1a\xa2\x01\n\x15\x43ommonDirectionConfig\x12\x39\n\x07\x65nabled\x18\x01 \x01(\x0b\x32(.envoy.config.core.v3.RuntimeFeatureFlag\x12\x38\n\x12min_content_length\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x14\n\x0c\x63ontent_type\x18\x03 \x03(\t\x1a~\n\x16RequestDirectionConfig\x12\x64\n\rcommon_config\x18\x01 \x01(\x0b\x32M.envoy.extensions.filters.http.compressor.v3.Compressor.CommonDirectionConfig\x1a\xa0\x02\n\x17ResponseDirectionConfig\x12\x64\n\rcommon_config\x18\x01 \x01(\x0b\x32M.envoy.extensions.filters.http.compressor.v3.Compressor.CommonDirectionConfig\x12\x1e\n\x16\x64isable_on_etag_header\x18\x02 \x01(\x08\x12%\n\x1dremove_accept_encoding_header\x18\x03 \x01(\x08\x12\x39\n\x1duncompressible_response_codes\x18\x04 \x03(\rB\x12\xfa\x42\x0f\x92\x01\x0c\x18\x01\"\x08*\x06\x10\xd8\x04(\xc8\x01\x12\x1d\n\x15status_header_enabled\x18\x05 \x01(\x08:8\x9a\xc5\x88\x1e\x33\n1envoy.config.filter.http.compressor.v2.Compressor\"_\n\x1aResponseDirectionOverrides\x12\x41\n\x1dremove_accept_encoding_header\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\"\xc9\x01\n\x13\x43ompressorOverrides\x12j\n\x19response_direction_config\x18\x01 \x01(\x0b\x32G.envoy.extensions.filters.http.compressor.v3.ResponseDirectionOverrides\x12\x46\n\x12\x63ompressor_library\x18\x02 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\"\x99\x01\n\x12\x43ompressorPerRoute\x12\x1b\n\x08\x64isabled\x18\x01 \x01(\x08\x42\x07\xfa\x42\x04j\x02\x08\x01H\x00\x12U\n\toverrides\x18\x02 \x01(\x0b\x32@.envoy.extensions.filters.http.compressor.v3.CompressorOverridesH\x00\x42\x0f\n\x08override\x12\x03\xf8\x42\x01\x42\xb7\x01\n9io.envoyproxy.envoy.extensions.filters.http.compressor.v3B\x0f\x43ompressorProtoP\x01Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/compressor/v3;compressorv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.compressor.v3.compressor_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n9io.envoyproxy.envoy.extensions.filters.http.compressor.v3B\017CompressorProtoP\001Z_github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/compressor/v3;compressorv3\272\200\310\321\006\002\020\002'
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG'].fields_by_name['uncompressible_response_codes']._loaded_options = None
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG'].fields_by_name['uncompressible_response_codes']._serialized_options = b'\372B\017\222\001\014\030\001\"\010*\006\020\330\004(\310\001'
  _globals['_COMPRESSOR'].fields_by_name['content_length']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['content_length']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['content_type']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['content_type']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['disable_on_etag_header']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['disable_on_etag_header']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['remove_accept_encoding_header']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['remove_accept_encoding_header']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['runtime_enabled']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['runtime_enabled']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_COMPRESSOR'].fields_by_name['compressor_library']._loaded_options = None
  _globals['_COMPRESSOR'].fields_by_name['compressor_library']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_COMPRESSOR']._loaded_options = None
  _globals['_COMPRESSOR']._serialized_options = b'\232\305\210\0363\n1envoy.config.filter.http.compressor.v2.Compressor'
  _globals['_COMPRESSORPERROUTE'].oneofs_by_name['override']._loaded_options = None
  _globals['_COMPRESSORPERROUTE'].oneofs_by_name['override']._serialized_options = b'\370B\001'
  _globals['_COMPRESSORPERROUTE'].fields_by_name['disabled']._loaded_options = None
  _globals['_COMPRESSORPERROUTE'].fields_by_name['disabled']._serialized_options = b'\372B\004j\002\010\001'
  _globals['_COMPRESSOR']._serialized_start=341
  _globals['_COMPRESSOR']._serialized_end=1608
  _globals['_COMPRESSOR_COMMONDIRECTIONCONFIG']._serialized_start=969
  _globals['_COMPRESSOR_COMMONDIRECTIONCONFIG']._serialized_end=1131
  _globals['_COMPRESSOR_REQUESTDIRECTIONCONFIG']._serialized_start=1133
  _globals['_COMPRESSOR_REQUESTDIRECTIONCONFIG']._serialized_end=1259
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG']._serialized_start=1262
  _globals['_COMPRESSOR_RESPONSEDIRECTIONCONFIG']._serialized_end=1550
  _globals['_RESPONSEDIRECTIONOVERRIDES']._serialized_start=1610
  _globals['_RESPONSEDIRECTIONOVERRIDES']._serialized_end=1705
  _globals['_COMPRESSOROVERRIDES']._serialized_start=1708
  _globals['_COMPRESSOROVERRIDES']._serialized_end=1909
  _globals['_COMPRESSORPERROUTE']._serialized_start=1912
  _globals['_COMPRESSORPERROUTE']._serialized_end=2065
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import errno
import os
import socket

_DEFAULT_SOCK_OPTIONS = (
    (socket.SO_REUSEADDR, socket.SO_REUSEPORT)
    if os.name != "nt"
    else (socket.SO_REUSEADDR,)
)
_UNRECOVERABLE_ERRNOS = (errno.EADDRINUSE, errno.ENOSR)


def get_socket(
    bind_address="localhost",
    port=0,
    listen=True,
    sock_options=_DEFAULT_SOCK_OPTIONS,
):
    """Opens a socket.

    Useful for reserving a port for a system-under-test.

    Args:
      bind_address: The host to which to bind.
      port: The port to which to bind.
      listen: A boolean value indicating whether or not to listen on the socket.
      sock_options: A sequence of socket options to apply to the socket.

    Returns:
      A tuple containing:
        - the address to which the socket is bound
        - the port to which the socket is bound
        - the socket object itself
    """
    _sock_options = sock_options if sock_options else []
    if socket.has_ipv6:
        address_families = (socket.AF_INET6, socket.AF_INET)
    else:
        address_families = socket.AF_INET
    for address_family in address_families:
        try:
            sock = socket.socket(address_family, socket.SOCK_STREAM)
            for sock_option in _sock_options:
                sock.setsockopt(socket.SOL_SOCKET, sock_option, 1)
            sock.bind((bind_address, port))
            if listen:
                sock.listen(1)
            return bind_address, sock.getsockname()[1], sock
        except OSError as os_error:
            sock.close()
            if os_error.errno in _UNRECOVERABLE_ERRNOS:
                raise
            else:
                continue
        # For PY2, socket.error is a child class of IOError; for PY3, it is
        # pointing to OSError. We need this catch to make it 2/3 agnostic.
        except socket.error:  # pylint: disable=duplicate-except
            sock.close()
            continue
    raise RuntimeError(
        "Failed to bind to {} with sock_options {}".format(
            bind_address, sock_options
        )
    )


@contextlib.contextmanager
def bound_socket(
    bind_address="localhost",
    port=0,
    listen=True,
    sock_options=_DEFAULT_SOCK_OPTIONS,
):
    """Opens a socket bound to an arbitrary port.

    Useful for reserving a port for a system-under-test.

    Args:
      bind_address: The host to which to bind.
      port: The port to which to bind.
      listen: A boolean value indicating whether or not to listen on the socket.
      sock_options: A sequence of socket options to apply to the socket.

    Yields:
      A tuple containing:
        - the address to which the socket is bound
        - the port to which the socket is bound
    """
    host, port, sock = get_socket(
        bind_address=bind_address,
        port=port,
        listen=listen,
        sock_options=sock_options,
    )
    try:
        yield host, port
    finally:
        sock.close()



# Copyright 2019 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The Python implementation of the TestServicer."""

import time

import grpc

from src.proto.grpc.testing import empty_pb2
from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc

_INITIAL_METADATA_KEY = "x-grpc-test-echo-initial"
_TRAILING_METADATA_KEY = "x-grpc-test-echo-trailing-bin"
_US_IN_A_SECOND = 1000 * 1000


def _maybe_echo_metadata(servicer_context):
    """Copies metadata from request to response if it is present."""
    invocation_metadata = dict(servicer_context.invocation_metadata())
    if _INITIAL_METADATA_KEY in invocation_metadata:
        initial_metadatum = (
            _INITIAL_METADATA_KEY,
            invocation_metadata[_INITIAL_METADATA_KEY],
        )
        servicer_context.send_initial_metadata((initial_metadatum,))
    if _TRAILING_METADATA_KEY in invocation_metadata:
        trailing_metadatum = (
            _TRAILING_METADATA_KEY,
            invocation_metadata[_TRAILING_METADATA_KEY],
        )
        servicer_context.set_trailing_metadata((trailing_metadatum,))


def _maybe_echo_status_and_message(request, servicer_context):
    """Sets the response context code and details if the request asks for them"""
    if request.HasField("response_status"):
        servicer_context.set_code(request.response_status.code)
        servicer_context.set_details(request.response_status.message)


class TestService(test_pb2_grpc.TestServiceServicer):
    def EmptyCall(self, request, context):
        _maybe_echo_metadata(context)
        return empty_pb2.Empty()

    def UnaryCall(self, request, context):
        _maybe_echo_metadata(context)
        _maybe_echo_status_and_message(request, context)
        return messages_pb2.SimpleResponse(
            payload=messages_pb2.Payload(
                type=messages_pb2.COMPRESSABLE,
                body=b"\x00" * request.response_size,
            )
        )

    def StreamingOutputCall(self, request, context):
        _maybe_echo_status_and_message(request, context)
        for response_parameters in request.response_parameters:
            if response_parameters.interval_us != 0:
                time.sleep(response_parameters.interval_us / _US_IN_A_SECOND)
            yield messages_pb2.StreamingOutputCallResponse(
                payload=messages_pb2.Payload(
                    type=request.response_type,
                    body=b"\x00" * response_parameters.size,
                )
            )

    def StreamingInputCall(self, request_iterator, context):
        aggregate_size = 0
        for request in request_iterator:
            if request.payload is not None and request.payload.body:
                aggregate_size += len(request.payload.body)
        return messages_pb2.StreamingInputCallResponse(
            aggregated_payload_size=aggregate_size
        )

    def FullDuplexCall(self, request_iterator, context):
        _maybe_echo_metadata(context)
        for request in request_iterator:
            _maybe_echo_status_and_message(request, context)
            for response_parameters in request.response_parameters:
                if response_parameters.interval_us != 0:
                    time.sleep(
                        response_parameters.interval_us / _US_IN_A_SECOND
                    )
                yield messages_pb2.StreamingOutputCallResponse(
                    payload=messages_pb2.Payload(
                        type=request.payload.type,
                        body=b"\x00" * response_parameters.size,
                    )
                )

    # NOTE(nathaniel): Apparently this is the same as the full-duplex call?
    # NOTE(atash): It isn't even called in the interop spec (Oct 22 2015)...
    def HalfDuplexCall(self, request_iterator, context):
        return self.FullDuplexCall(request_iterator, context)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/api_key_auth/v3/api_key_auth.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/api_key_auth/v3/api_key_auth.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import sensitive_pb2 as udpa_dot_annotations_dot_sensitive__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n@envoy/extensions/filters/http/api_key_auth/v3/api_key_auth.proto\x12-envoy.extensions.filters.http.api_key_auth.v3\x1a\x1fxds/annotations/v3/status.proto\x1a udpa/annotations/sensitive.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\x82\x02\n\nApiKeyAuth\x12V\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.CredentialB\x06\xb8\xb7\x8b\xa4\x02\x01\x12M\n\x0bkey_sources\x18\x02 \x03(\x0b\x32\x38.envoy.extensions.filters.http.api_key_auth.v3.KeySource\x12M\n\nforwarding\x18\x03 \x01(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.Forwarding\"\xa3\x02\n\x12\x41piKeyAuthPerRoute\x12V\n\x0b\x63redentials\x18\x01 \x03(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.CredentialB\x06\xb8\xb7\x8b\xa4\x02\x01\x12M\n\x0bkey_sources\x18\x02 \x03(\x0b\x32\x38.envoy.extensions.filters.http.api_key_auth.v3.KeySource\x12\x17\n\x0f\x61llowed_clients\x18\x03 \x03(\t\x12M\n\nforwarding\x18\x04 \x01(\x0b\x32\x39.envoy.extensions.filters.http.api_key_auth.v3.Forwarding\";\n\nCredential\x12\x14\n\x03key\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x17\n\x06\x63lient\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\"j\n\tKeySource\x12!\n\x06header\x18\x01 \x01(\tB\x11\xfa\x42\x0er\x0c\x18\x80\x08\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\x12\x17\n\x05query\x18\x02 \x01(\tB\x08\xfa\x42\x05r\x03\x18\x80\x08\x12!\n\x06\x63ookie\x18\x03 \x01(\tB\x11\xfa\x42\x0er\x0c\x18\x80\x08\xc0\x01\x01\xc8\x01\x00\xd0\x01\x01\"F\n\nForwarding\x12\x1e\n\x06header\x18\x01 \x01(\tB\x0e\xfa\x42\x0br\t\x18\x80\x08\xc0\x01\x01\xc8\x01\x00\x12\x18\n\x10hide_credentials\x18\x02 \x01(\x08\x42\xc5\x01\n;io.envoyproxy.envoy.extensions.filters.http.api_key_auth.v3B\x0f\x41piKeyAuthProtoP\x01Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/api_key_auth/v3;api_key_authv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.api_key_auth.v3.api_key_auth_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n;io.envoyproxy.envoy.extensions.filters.http.api_key_auth.v3B\017ApiKeyAuthProtoP\001Zcgithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/api_key_auth/v3;api_key_authv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_APIKEYAUTH'].fields_by_name['credentials']._loaded_options = None
  _globals['_APIKEYAUTH'].fields_by_name['credentials']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_APIKEYAUTHPERROUTE'].fields_by_name['credentials']._loaded_options = None
  _globals['_APIKEYAUTHPERROUTE'].fields_by_name['credentials']._serialized_options = b'\270\267\213\244\002\001'
  _globals['_CREDENTIAL'].fields_by_name['key']._loaded_options = None
  _globals['_CREDENTIAL'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_CREDENTIAL'].fields_by_name['client']._loaded_options = None
  _globals['_CREDENTIAL'].fields_by_name['client']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_KEYSOURCE'].fields_by_name['header']._loaded_options = None
  _globals['_KEYSOURCE'].fields_by_name['header']._serialized_options = b'\372B\016r\014\030\200\010\300\001\001\310\001\000\320\001\001'
  _globals['_KEYSOURCE'].fields_by_name['query']._loaded_options = None
  _globals['_KEYSOURCE'].fields_by_name['query']._serialized_options = b'\372B\005r\003\030\200\010'
  _globals['_KEYSOURCE'].fields_by_name['cookie']._loaded_options = None
  _globals['_KEYSOURCE'].fields_by_name['cookie']._serialized_options = b'\372B\016r\014\030\200\010\300\001\001\310\001\000\320\001\001'
  _globals['_FORWARDING'].fields_by_name['header']._loaded_options = None
  _globals['_FORWARDING'].fields_by_name['header']._serialized_options = b'\372B\013r\t\030\200\010\300\001\001\310\001\000'
  _globals['_APIKEYAUTH']._serialized_start=239
  _globals['_APIKEYAUTH']._serialized_end=497
  _globals['_APIKEYAUTHPERROUTE']._serialized_start=500
  _globals['_APIKEYAUTHPERROUTE']._serialized_end=791
  _globals['_CREDENTIAL']._serialized_start=793
  _globals['_CREDENTIAL']._serialized_end=852
  _globals['_KEYSOURCE']._serialized_start=854
  _globals['_KEYSOURCE']._serialized_end=960
  _globals['_FORWARDING']._serialized_start=962
  _globals['_FORWARDING']._serialized_end=1032
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import make_grpcio_tools as _make

OUT_OF_DATE_MESSAGE = """file {} is out of date

Have you called tools/distrib/python/make_grpcio_tools.py since upgrading protobuf?"""

submodule_commit_hash = _make.protobuf_submodule_commit_hash()

with open(_make.GRPC_PYTHON_PROTOC_LIB_DEPS, "r") as _protoc_lib_deps_file:
    content = _protoc_lib_deps_file.read().splitlines()

testString = (
    _make.COMMIT_HASH_PREFIX + submodule_commit_hash + _make.COMMIT_HASH_SUFFIX
)

if testString not in content:
    print(OUT_OF_DATE_MESSAGE.format(_make.GRPC_PYTHON_PROTOC_LIB_DEPS))
    raise SystemExit(1)
